/***************************************************************
 search.m

 MAGMA code to seive for all f of level <= 1000 that satify
 both conditions 1 and 2, given the table 
       http://modular.fas.harvard.edu/Tables/an_1-1000.gp

 William Stein and David Savitt
 19 February 2001.

 ***************************************************************/

function SatisfiesConditionOne(N, f)
   return Degree(f[1]) eq 2;
end function;

function SatisfiesConditionTwoAtEll(N, f, ell)
   if not IsPrime(ell) or ell gt 5 then 
      return false;
   end if;
   if N mod ell^2 ne 0 then
      return false;
   end if;
   if N mod ell^3 eq 0 then
      return false;
   end if;

   K := NumberField(f[1]);
   O := MaximalOrder(K);
   I := ideal<O | ell>;
   F := Factorization(I);
   if #F eq 1 and F[1][2] eq 1 then
      return false;
   end if;

   return true;
end function;

function SatisfiesConditionTwo(N, f)
   return [ell : ell in [3,5] | SatisfiesConditionTwoAtEll(N, f,ell)];
end function;

function SatisfiesConditionThree(N, iso, ell, prec)

   if not (ell in {3,5}) then
      error "SatisfiesConditionThree: not yet programmed.";
   end if;

   S := CuspForms(N,2);
   A := NewformDecomposition(S)[iso];
   "f = ", qExpansion(Newforms(A)[1],8);
   Modulus(BaseRing(Parent(qExpansion(Newforms(A)[1],8))));
   assert Dimension(A) eq 2;
   A_mod_ell := BaseExtend(A,GF(ell));
   "fbar = ", qExpansion(Newforms(A_mod_ell), prec);   
   works := false;
   for f in qExpansion(Newforms(A_mod_ell), prec) do
     "ap   = ", [<p, Coefficient(f,p)> : p in [2..prec-1] | IsPrime(p) and N mod p ne 0];
      case ell:
         when 3:
            ap := [Coefficient(f,p) : p in [2..prec-1] | 
                   IsPrime(p) and p mod 3 eq 2 and (N mod p ne 0) ];
             if Index(ap,0) ne 0 and 
                   (Index(ap,1) ne 0 or Index(ap,2) ne 0) then
               works := true; 
               print "This works.";
            end if;
         when 5:
            /*
            l=5:  we wish to find a prime p congruent to +/-1 (mod 5) such
            that a_p = 0 (mod 5), and a prime p' satisfying any of the
            following
            --  p' =  1 (mod 5) with a_p' = +/- 1 (mod 5)
            --  p' = -1 (mod 5) with a_p' = +/- 2 (mod 5)
            --  p' = +/-2 (mod 5) with a_p' <> 0 (mod 5).
            */
            // find p:
            P := [p : p in [2..prec-1] | 
                   N mod p ne 0 and IsPrime(p) and 
                    ((p mod 5) in {+1,4}) and Coefficient(f,p) eq 0];
            if #P ge 1 then
               print "The p are ", P;
               // find p':
               PP := [p : p in [2..prec-1] | 
                      N mod p ne 0 and IsPrime(p) and 
                    (  (p mod 5 eq 1 and Coefficient(f,p) in {GF(5)!1,4}) or
                       (p mod 5 eq 4 and Coefficient(f,p) in {GF(5)!2,3}) or
                       (p mod 5 in {2,3} and Coefficient(f,p) ne 0) )];
               if #PP ge 1 then
                  print "The p' are ", PP;
                  works := true;
               end if;
            else
               print "there is no p.";
            end if;
         else:
            error "Case ell = ", ell, " of condition 3 not yet programmed.";               
      end case;
   end for;
   return works;
end function;

procedure ListExamples(Nrange, table)
   printf "examples := [\n";
   for N in Nrange do
      for A in table[N] do
         f := <A[3], A[4]>;
         if SatisfiesConditionOne(N, f) then
            two := SatisfiesConditionTwo(N,f);
            if #two gt 0 then
               printf "<%o, %o, %o, %o>,\n", N, two, A[1], Discriminant(A[3]);
            end if;
         end if;
      end for;
   end for;
   printf "];\n";
end procedure;

function ModularKernels(examples)
   i := 1;
   ans := "";
   while i le #examples do
      A := examples[i][2];
      N := A[1];
      M := ModularSymbols(N);
      D := SortDecomposition(NewformDecomposition(
                      NewSubspace(CuspidalSubspace(M)))
           );
      while A[1] eq N do
         ans := ans cat Sprintf("ModularKernel[%o] := %o; // N = %o\n", examples[i][1], 
				Invariants(ModularKernel(D[A[3]])), N);
         i +:= 1;
         if i gt #examples then
            break;
         end if;
         A := examples[i][2];
      end while;
      DisownChildren(M);
   end while;
   return ans;
end function;


