/***************************************************************
 search.m

 MAGMA code to seive for all f of level <= 1000 that satify
 both conditions 1 and 2, given the table 
       http://modular.fas.harvard.edu/Tables/an_1-1000.gp

 William Stein and David Savitt
 19 February 2001.

 ***************************************************************/

function SatisfiesConditionOne(N, f)
   return Degree(f[1]) eq 2;
end function;

function SatisfiesConditionTwoAtEll(N, f, ell)
   if not IsPrime(ell) or ell gt 5 then 
      return false;
   end if;
   if N mod ell^2 ne 0 then
      return false;
   end if;
   if N mod ell^3 eq 0 then
      return false;
   end if;

   K := NumberField(f[1]);
   O := MaximalOrder(K);
   I := ideal<O | ell>;
   F := Factorization(I);
   if #F eq 1 and F[1][2] eq 1 then
      return false;
   end if;

   return true;
end function;

function SatisfiesConditionTwo(N, f)
   return [ell : ell in [3,5] | SatisfiesConditionTwoAtEll(N, f,ell)];
end function;

function SatisfiesConditionThree(N, iso, ell, prec)

   if not (ell in {3,5}) then
      error "SatisfiesConditionThree: not yet programmed.";
   end if;

   S := CuspForms(N,2);
   A := NewformDecomposition(S)[iso];
   "f = ", qExpansion(Newforms(A)[1],8);
   Modulus(BaseRing(Parent(qExpansion(Newforms(A)[1],8))));
   assert Dimension(A) eq 2;
   A_mod_ell := BaseExtend(A,GF(ell));
   "fbar = ", qExpansion(Newforms(A_mod_ell), prec);   
   works := false;
   for f in qExpansion(Newforms(A_mod_ell), prec) do
     "ap   = ", [<p, Coefficient(f,p)> : p in [2..prec-1] | IsPrime(p) and N mod p ne 0];
      case ell:
         when 3:
            ap := [Coefficient(f,p) : p in [2..prec-1] | 
                   IsPrime(p) and p mod 3 eq 2 and (N mod p ne 0) ];
             if Index(ap,0) ne 0 and 
                   (Index(ap,1) ne 0 or Index(ap,2) ne 0) then
               works := true; 
               print "This works.";
            end if;
         when 5:
            /*
            l=5:  we wish to find a prime p congruent to +/-1 (mod 5) such
            that a_p = 0 (mod 5), and a prime p' satisfying any of the
            following
            --  p' =  1 (mod 5) with a_p' = +/- 1 (mod 5)
            --  p' = -1 (mod 5) with a_p' = +/- 2 (mod 5)
            --  p' = +/-2 (mod 5) with a_p' <> 0 (mod 5).
            */
            // find p:
            P := [p : p in [2..prec-1] | 
                   N mod p ne 0 and IsPrime(p) and 
                    ((p mod 5) in {+1,4}) and Coefficient(f,p) eq 0];
            if #P ge 1 then
               print "The p are ", P;
               // find p':
               PP := [p : p in [2..prec-1] | 
                      N mod p ne 0 and IsPrime(p) and 
                    (  (p mod 5 eq 1 and Coefficient(f,p) in {GF(5)!1,4}) or
                       (p mod 5 eq 4 and Coefficient(f,p) in {GF(5)!2,3}) or
                       (p mod 5 in {2,3} and Coefficient(f,p) ne 0) )];
               if #PP ge 1 then
                  print "The p' are ", PP;
                  works := true;
               end if;
            else
               print "there is no p.";
            end if;
         else:
            error "Case ell = ", ell, " of condition 3 not yet programmed.";               
      end case;
   end for;
   return works;
end function;


/*  4) rhobar is modular  (this is automatic, since ell=3, 5.)  */
function SatisfiesConditionFour(N, iso, ell)
   if ell in {3, 5} then
      return true;
   end if;
   error "We don't know enough of Serre's.";
end function;


/*
  5) The newform f is not equal to g tensor chi for g of lower 
   level and chi is of conductor ell.
*/

function SatisfiesConditionFive(N, iso, ell, prec)

   // 1. Compute the chi's Z / ell Z --> C
   K := CyclotomicField(ell-1);
   dirgroup := DirichletGroup(ell, K);
   G := [e : e in Elements(dirgroup) | e ne dirgroup!1];

   // 0. Compute f.
   S := CuspForms(N,2);
   A := NewformDecomposition(S)[iso];
   f := qExpansion(Newforms(A)[1],prec);
   "f = ", f;
   Parent(qExpansion(Newforms(A)[1],8));
   R := PolynomialRing(RationalField());
   S := PolynomialRing(K);
   modulus := S!Modulus(BaseRing(Parent(f)));
   assert IsIrreducible(modulus);
   T := quo<S|modulus>;
   U<q> := PowerSeriesRing(T);
   f := &+[q^n * (T!(S!(R!Coefficient(f,n)))) : n in [0..prec-1]];

   // 2. For each nonidentity character chi, determine
   // whether or not there is a form g of level M strictly dividing
   // N and of nebentypus chi^(-2) such that 
   //              a_p(g)*chi(p) = a_p(f)
   // for all p not dividing ell*N and p < prec.

   // (MAYBE THE M SHOULD BE FURTHER RESTRICTED?!)
   for M in [n : n in Divisors(N) | n lt N and n gt 1] do
      for chi in G do
         if M mod Conductor(chi^(-2)) eq 0 then
            if Conductor(chi^(-2)) eq 1 then
               S := CuspidalSubspace(ModularSymbols(M,2));
            else
               S := CuspidalSubspace(ModularSymbols(Extend(chi^(-2),M),2));
            end if;
           
            "Consider S = ", S;
/*            for z in NewformDecomposition(NewSubspace(CuspidalSubspace(S))) do
               qexp := qEigenform(z,prec);
               print qexp;
               if Type(BaseRing(Parent(qexp))) eq RngUPolRes then
                  print Modulus(BaseRing(Parent(qexp))), "  disc = ", Discriminant(Modulus(BaseRing(Parent(qexp))));
               end if;
            end for;
*/
            if Dimension(S) gt 0 then
               // Assume the worst, i.e., that g exists.  Let's find g.
               // If we fail, then g does not exist. 
               // Compute the characteristic polynomial of 
               //           a_p(g) = a_p(f)*chi^(-1)(p).
               I := [<p,MinimalPolynomial(Coefficient(f,p)*Evaluate(chi^(-1),p))> : 
                             p in [2..prec-1] | N*ell mod p ne 0 and IsPrime(p)];
"I = ", I;
               V := Kernel(I,S);
               if Dimension(V) gt 0 then   // ut oh!!
                  print "FOUND g!!!";
                  print "M = ", M;
                  print "chi = ", chi;
                  return false;
               end if;
            end if;
         end if;
      end for;
   end for;
   return true;
end function;


function SatisfiesConditionSix(N, iso, ell, which_prime, prec)
   S := CuspForms(N,2);
   A := NewformDecomposition(S)[iso];
   "f = ", qExpansion(Newforms(A)[1],8);
   Modulus(BaseRing(Parent(qExpansion(Newforms(A)[1],8))));
   assert Dimension(A) eq 2;
   A_mod_ell := BaseExtend(A,GF(ell));
   f := qExpansion(Newforms(A_mod_ell), prec);
   "f = ", f;

   /* QUESTION: Let g be a form of level N/(ell^(ord_ell(N))) that is congruent 
      to f modulo the prime over ell that we are considering.   Is g ordinary 
      or supersingular at ell?
    */

   return false;
end function;


procedure ListExamples(Nrange, table)
   printf "examples := [\n";
   for N in Nrange do
      for A in table[N] do
         f := <A[3], A[4]>;
         if SatisfiesConditionOne(N, f) then
            two := SatisfiesConditionTwo(N,f);
            if #two gt 0 then
               printf "<%o, %o, %o, %o>,\n", N, two, A[1], Discriminant(A[3]);
            end if;
         end if;
      end for;
   end for;
   printf "];\n";
end procedure;

function ModularKernels(examples)
   i := 1;
   ans := "";
   while i le #examples do
      A := examples[i][2];
      N := A[1];
      M := ModularSymbols(N);
      D := SortDecomposition(NewformDecomposition(
                      NewSubspace(CuspidalSubspace(M)))
           );
      while A[1] eq N do
         ans := ans cat Sprintf("ModularKernel[%o] := %o; // N = %o\n", examples[i][1], 
				Invariants(ModularKernel(D[A[3]])), N);
         i +:= 1;
         if i gt #examples then
            break;
         end if;
         A := examples[i][2];
      end while;
      DisownChildren(M);
   end while;
   return ans;
end function;


/* f is the q-expansion of a mod-ell newform of level N with
   coefficients in Z/ell Z.
   This function returns pairs <k, eps, chi> such
   that there is mod-ell form g of weight k and character eps such
   that g x chi = f..
*/
function Space(N, iso)
   S := CuspForms(N,2);
   A := NewformDecomposition(S)[iso];
   return A;
end function;

function FindTwistInLowWeight(f, N)
   ell := Characteristic(BaseRing(Parent(f)));
   assert N mod ell ne 0;
   prec := AbsolutePrecision(f);
   n   := EulerPhi(N) div (ell^Valuation(EulerPhi(N),ell));
   pow := Order(IntegerRing(n)!ell);  // ord divides ell^pow - 1 
   F<alp> := GF(ell^pow);
   G := DirichletGroup(N,F);
   Cyclo := DirichletGroup(ell,F);

  /* This implementation is VERY stupid.  However, I trust it, whereas
     I don't trust computing kernels at present. */
   ans := [];
   for eps in Elements(G) do
      print "eps = ", eps;
      for k in [2..ell+1] do
         print "k = ", k;
         S := CuspForms(eps,k);
         for g in qExpansion(Newforms(S),prec) do
            print "g = ", qExpansion(Newforms(S), 6);
            if Type(BaseRing(Parent(g))) eq RngUPolRes then
               break;
            end if;
            for chi in Elements(Cyclo) do
               equal := true;
               for p in [q : q in [2..prec-1] | IsPrime(q) and N mod q ne 0 and q ne ell] do 
                  if Coefficient(f,p)*Evaluate(chi,p) ne Coefficient(g,p) then
                     equal := false;
                     break;
                  end if;
               end for;
               if equal then
                  Append(~ans, <k, eps, chi, g>);
               end if;
            end for;
         end for;
      end for;
   end for;
   return ans;
end function;

procedure ConditionSix(prec)
//   ex := [[175,5,5], [275,7,5], [575,7,5], [693,5,3]];
//   ex := [[275,7,5], [575,7,5], [693,5,3]];
   ex := [[693,5,3]];
   for e in ex do
      print "\n\n\n*******************\nLEVEL N = ", ex[1];
      A := BaseExtend(Space(e[1],e[2]),GF(e[3]));
      for f in qExpansion(Newforms(A),prec) do
         print "Consider newform f = ", f;
         print "FindTwist = ", FindTwistInLowWeight(f,e[1] div (e[3]^2));
      end for;
   end for;
end procedure;
